@@ -8,6 +8,9 @@ if (has_dev_api()) {
   assignInNamespace("var_names", finbif:::var_names_test, "finbif")
   assignInNamespace("filter_names", finbif:::filter_names_test, "finbif")
   assignInNamespace("has_value", finbif:::has_value_test, "finbif")
+  assignInNamespace(
+    "lite_download_file_vars", finbif:::lite_download_file_vars_test, "finbif"
+  )
 
   invisible(vcr_configure(dir = "../api-dev-cassettes"))
 
